#!/usr/bin/env python3
#******************************************************************************
# Copyright (C) 2018 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
# This file is part of the textoutpc project, which is MIT-licensed.
#******************************************************************************
""" Unit tests for the `textoutpc` Python module. """

# This file is only there to indicate that the folder is a module.
# It doesn't actually contain code.

# End of file.
